#region Copyright Syncfusion Inc. 2001 -2004
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
//  Copyright Syncfusion Inc. All rights reserved. Use of this code is subject to the terms of our 
//  license. A copy of the current license can be obtained at any time by e-mailing licensing@syncfusion.com. 
//  Re-distribution in any form is strictly prohibited. Any infringement will be prosecuted under applicable laws. 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
#endregion 
 
namespace TabbedMDI {

	using System;
	using System.ComponentModel;
	using System.Drawing;
	using System.Windows.Forms;
	using Syncfusion.Windows.Forms;
	using Microsoft.Win32;
	using Syncfusion.Windows.Forms.Tools;
	using Syncfusion.Windows.Forms.Tools.XPMenus;
	using Syncfusion.Runtime.Serialization;

	using System.Diagnostics;
	using System.Runtime.InteropServices;

    public class MainForm : System.Windows.Forms.Form {
      private System.ComponentModel.IContainer components;
        protected System.Windows.Forms.StatusBar statusBar1;
		private System.Windows.Forms.MdiClient mdiClient1;
		private TabbedMDIManager tabbedMDIManager;
		private MenuItem miWindow;
      private Syncfusion.Windows.Forms.Tools.XPMenus.MainFrameBarManager mainFrameBarManager1;
      private Syncfusion.Windows.Forms.Tools.XPMenus.MdiListBarItem mdiListBarItem1;
      private Syncfusion.Windows.Forms.Tools.XPMenus.Bar bar1;
      private Syncfusion.Windows.Forms.Tools.XPMenus.ParentBarItem parentBarItem1;
		private bool tabWindowsOn = false;

		// <samplecodeblock name="Initializing TabbedMDIManager">
        public MainForm() {
            
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

			tabbedMDIManager = new TabbedMDIManager();
			//tabbedMDIManager.UseIconsInTabs = false;
			
			// <samplecodeblock name="TabbedMDIManager: Setting the Context Menu">
			// Append menus to the standard mdi tab context menu
			ParentBarItem contextMenuItem = new ParentBarItem();

			BarItem newDocItem = new BarItem();
			newDocItem.Click += new System.EventHandler(this.addDoc1_Click);
			newDocItem.Text = "Custom Item: Insert New Doc";
			newDocItem.MergeOrder = 30;
			contextMenuItem.Items.Add(newDocItem);

			BarItem exitItem = new BarItem();
			exitItem.Click += new System.EventHandler(this.FileExit_Clicked);
			exitItem.Text = "CustomItem: Exit";
			exitItem.MergeOrder = 30;
			contextMenuItem.Items.Add(exitItem);

			contextMenuItem.BeginGroupAt(newDocItem);
			// Items in this ParentBarItem will be merged with the standard context menu ParentBarItem of the mdi tab.
			tabbedMDIManager.ContextMenuItem = contextMenuItem;
			//</samplecodeblock>
        }

		// Convenient way to toggle TabbedMDI mode.
		internal bool TabbedMDIOn
		{
			get	{	return this.tabWindowsOn;	}
			set
			{
				if(!(this.tabWindowsOn == value))
				{
					this.tabWindowsOn = value;

					if(this.tabWindowsOn)
					{
						this.tabbedMDIManager.AttachToMdiContainer(this);
					}
					else
					{
						this.tabbedMDIManager.DetachFromMdiContainer(this, false); // false to not invoke the Cascade mode after detaching.
					}
				}
			}
		}

		internal TabbedMDIManager TabbedMDIManager
		{
			get{return this.tabbedMDIManager;}
		}

        //Add a document 
        private void AddDocument(Form doc) {
            doc.MdiParent = this;
            doc.Show();
        }

		private void MainForm_Load(object sender, System.EventArgs e)
		{
			// Add 4 documents
			this.addDoc1_Click(this, EventArgs.Empty);
			this.addDoc1_Click(this, EventArgs.Empty);
			this.addDoc1_Click(this, EventArgs.Empty);
			this.addDoc1_Click(this, EventArgs.Empty);

			// Turn on MDI Tabbed Documents mode.
			// Call this after loading the mdi children to restore their previous state.
			this.TabbedMDIOn = true;
		}

		private int document1Count = 0 ;
		internal void addDoc1_Click(object sender, System.EventArgs e)
		{
			document1Count++ ;
			Document1 doc = new Document1("DocumentOne " + document1Count.ToString());
			AddDocument(doc);
			this.tabbedMDIManager.SetTooltip(doc, "Tooltip for " + doc.Text);
		}

        //Window->Cascade Menu item handler
        protected internal void WindowCascade_Clicked(object sender, System.EventArgs e) {
			this.TabbedMDIOn = false;
            this.LayoutMdi(MdiLayout.Cascade);   
        }
		// </samplecodeblock>

        //Window->Tile Horizontally Menu item handler
        protected void WindowTileH_Clicked(object sender, System.EventArgs e) {
			this.TabbedMDIOn = false;
            this.LayoutMdi(MdiLayout.TileHorizontal);
        }


        //Window->Tile Vertically Menu item handler
        protected void WindowTileV_Clicked(object sender, System.EventArgs e) {
			this.TabbedMDIOn = false;
            this.LayoutMdi(MdiLayout.TileVertical);
        }

		protected internal void TabbedWindows_Clicked(object sender, System.EventArgs e)
		{
			this.TabbedMDIOn = true;
		}

		//File->Exit Menu item handler
		protected void FileExit_Clicked(object sender, System.EventArgs e) 
		{
			this.Close();
		}

		//One of the MDI Child windows has been activated
		protected void MDIChildActivated(object sender, System.EventArgs e) 
		{
			if (this.ActiveMdiChild == null) 
			{
				statusBar1.Text = "";
			} 
			else 
			{
				statusBar1.Text = this.ActiveMdiChild.Text;
			}
		}

        /// <summary>
        ///    Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing) 
        {
            base.Dispose(disposing);
			if(components != null)
				components.Dispose();
        }


        /// <summary>
        ///    Required method for Designer support - do not modify
        ///    the contents of this method with the code editor.
        /// </summary>
        protected void InitializeComponent()
        {
          this.components = new System.ComponentModel.Container();
          System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
          this.mdiClient1 = new System.Windows.Forms.MdiClient();
          this.statusBar1 = new System.Windows.Forms.StatusBar();
          this.mainFrameBarManager1 = new Syncfusion.Windows.Forms.Tools.XPMenus.MainFrameBarManager(this.components, this);
          this.bar1 = new Syncfusion.Windows.Forms.Tools.XPMenus.Bar(this.mainFrameBarManager1, "Menu");
          this.parentBarItem1 = new Syncfusion.Windows.Forms.Tools.XPMenus.ParentBarItem();
          this.mdiListBarItem1 = new Syncfusion.Windows.Forms.Tools.XPMenus.MdiListBarItem();
          ((System.ComponentModel.ISupportInitialize)(this.mainFrameBarManager1)).BeginInit();
          this.SuspendLayout();
          // 
          // mdiClient1
          // 
          this.mdiClient1.Dock = System.Windows.Forms.DockStyle.Fill;
          this.mdiClient1.Location = new System.Drawing.Point(0, 26);
          this.mdiClient1.Name = "mdiClient1";
          this.mdiClient1.TabIndex = 4;
          // 
          // statusBar1
          // 
          this.statusBar1.Location = new System.Drawing.Point(0, 321);
          this.statusBar1.Name = "statusBar1";
          this.statusBar1.Size = new System.Drawing.Size(624, 24);
          this.statusBar1.TabIndex = 1;
          // 
          // mainFrameBarManager1
          // 
          this.mainFrameBarManager1.BarPositionInfo = ((System.IO.MemoryStream)(resources.GetObject("mainFrameBarManager1.BarPositionInfo")));
          this.mainFrameBarManager1.Bars.Add(this.bar1);
          this.mainFrameBarManager1.Categories.Add("Menu");
          this.mainFrameBarManager1.CurrentBaseFormType = "System.Windows.Forms.Form";
          this.mainFrameBarManager1.Form = this;
          this.mainFrameBarManager1.ImageList = null;
          this.mainFrameBarManager1.Items.AddRange(new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem[] {
                                                                                                          this.mdiListBarItem1,
                                                                                                          this.parentBarItem1});
          this.mainFrameBarManager1.LargeImageList = null;
          this.mainFrameBarManager1.ResetCustomization = false;
          // 
          // bar1
          // 
          this.bar1.BarName = "Menu";
          this.bar1.BarStyle = ((Syncfusion.Windows.Forms.Tools.XPMenus.BarStyle)((((Syncfusion.Windows.Forms.Tools.XPMenus.BarStyle.AllowQuickCustomizing | Syncfusion.Windows.Forms.Tools.XPMenus.BarStyle.IsMainMenu) 
            | Syncfusion.Windows.Forms.Tools.XPMenus.BarStyle.Visible) 
            | Syncfusion.Windows.Forms.Tools.XPMenus.BarStyle.DrawDragBorder)));
          this.bar1.Items.AddRange(new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem[] {
                                                                                          this.parentBarItem1});
          this.bar1.Manager = this.mainFrameBarManager1;
          // 
          // parentBarItem1
          // 
          this.parentBarItem1.CategoryIndex = 0;
          this.parentBarItem1.ID = "Windows";
          this.parentBarItem1.Items.AddRange(new Syncfusion.Windows.Forms.Tools.XPMenus.BarItem[] {
                                                                                                    this.mdiListBarItem1});
          this.parentBarItem1.Text = "Syncfusion MDI &Windows";
          // 
          // mdiListBarItem1
          // 
          this.mdiListBarItem1.CategoryIndex = 0;
          this.mdiListBarItem1.ID = "Window List";
          // 
          // MainForm
          // 
          this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
          this.ClientSize = new System.Drawing.Size(624, 345);
          this.Controls.Add(this.mdiClient1);
          this.Controls.Add(this.statusBar1);
          this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
          this.IsMdiContainer = true;
          this.Name = "MainForm";
          this.Text = "Tabbed MDI (Syncfusion Inc.)";
          this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
          this.MdiChildActivate += new System.EventHandler(this.MDIChildActivated);
          this.Load += new System.EventHandler(this.MainForm_Load);
          ((System.ComponentModel.ISupportInitialize)(this.mainFrameBarManager1)).EndInit();
          this.ResumeLayout(false);

        }

		#region DLL Imports
		[DllImport("user32.dll", CharSet=CharSet.Auto)]
		extern public static bool SetForegroundWindow(IntPtr hwnd);
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		public static void Main() 
		{
			// Modify this version, whenever changes are made such that the persisted state cannot be loaded anymore.
			string curVersion = "1.0";
			ValidatePersistedStateVersion(curVersion);
			StoreVersionInfo(curVersion);

			MainForm mainForm = new MainForm();

			bool bRunning = false;
			foreach ( Process p in Process.GetProcesses(System.Environment.MachineName) )
			{
				if( p.MainWindowHandle != IntPtr.Zero)
				{
					// Process name gets truncated!
					if(p.ProcessName == "TabbedMDI")
					{
						bRunning = true;
						MessageBox.Show("Activating the already open instance of TabbedMDI.exe.");
						SetForegroundWindow(p.MainWindowHandle);
						break;
					}
				}
			}
			if(!bRunning)
			{
				Application.Run(mainForm);
			}
		}

		#region VERSIONING
		static string VersionTag = "AppVersion";
		static void ValidatePersistedStateVersion(string curVersion)
		{
			// Versioning
			AppStateSerializer serializer = AppStateSerializer.GetSingleton();
			object objectOut = serializer.DeserializeObject(VersionTag);
			if(objectOut != null)
			{
				string version = (string)objectOut;
				version = version.ToLower();

				curVersion = curVersion.ToLower();

				// Persisted info is not the current version, lets lose it.
				if(version != curVersion)
					serializer.FlushSerializer();
			}
		}
		// Store the current version, so that we can validate it the next time.
		static void StoreVersionInfo(string curVersion)
		{
			AppStateSerializer serializer = AppStateSerializer.GetSingleton();
			serializer.SerializeObject(VersionTag, curVersion, true);
		}
		#endregion VERSIONING

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			this.TabbedMDIOn = false;
		}

		private void parseChildren_Click(object sender, System.EventArgs e)
		{
			string children = String.Empty;
			// <samplecodeblock name="TabbedMDIManager: Getting the mdi children">
			foreach(Form form in this.tabbedMDIManager.MdiChildren)
			{
				children += form.Text + "\r\n";
			}
			//</samplecodeblock>
			MessageBox.Show(children);	
		}

		private void toggleAlign_Click(object sender, System.EventArgs e)
		{
			this.tabbedMDIManager.Horizontal = !this.tabbedMDIManager.Horizontal;
		}

		#region AboutForm_Logic
		DemoCommon.AboutForm aboutForm = null;
		private void menuItem4_Click(object sender, System.EventArgs e)
		{
			aboutForm = new DemoCommon.AboutForm();
			aboutForm.label1.Text = "Syncfusion Essential Tools - TabbedMDI Sample";
			aboutForm.button1.Click += new EventHandler(this.AboutFormCloseBtnClicked);
			aboutForm.AcceptButton = aboutForm.button1;
			aboutForm.ShowDialog();
		}
		private void AboutFormCloseBtnClicked(object sender, EventArgs e)
		{
			aboutForm.Close();
		}
		#endregion AboutForm_Logic
    }
}
